﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Domain.DTO;

namespace Domain.Implementation
{
   public static class DataAttributeRepository
    {
       public static DataAttribute GetById(int id)
       {
           if (id == 0) return null;
           using (var context = new DomainDbContext())
           {
               return context.DataAttributes.Include(x => x.AttributeType).FirstOrDefault(x => x.Id == id);
           }
       }

       public static DataAttribute Add(int datasetId, int attributeId, string value)
       {
           if (value == "") return null;
           using (var context = new DomainDbContext())
           {
               var dataSet = context.DataSets.FirstOrDefault(x => x.Id == datasetId);
               if (dataSet == null) return null;
               var attributeType = context.AttributeTypes.FirstOrDefault(x => x.Id == attributeId);
               if (attributeType == null) return null;
               var addedObject = context.DataAttributes.Add(new DataAttribute()
               {
                   Value = value,
                   DataSet = dataSet,
                   AttributeType = attributeType

               });
               context.SaveChanges();
               return addedObject;
           }
       }

       public static List<RelatedAttributesSimulated> GetAttributeRelations(List<int> attributes)
       {
           if (attributes == null) return new List<RelatedAttributesSimulated>();
           if (!attributes.Any()) return new List<RelatedAttributesSimulated>();
           using (var context = new DomainDbContext())
           {
               var returnList = new List<RelatedAttributesSimulated>();
               foreach(var attr in attributes){
                   returnList.AddRange(context.RelatedAttributesSimulated.Where(x => x.AttributeId == attr));
               }
               return returnList;
           }
       }
    }
}
