﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Domain.DTO;

namespace Domain.Implementation
{
   public static class DataCaseRepository
    {
       public static DataCase GetById(int id)
       {
           if (id == 0) return null;
           using (var context = new DomainDbContext())
           {
               return context.DataCases.Include(x=>x.Attributes).Include(x=>x.DataClass).FirstOrDefault(x => x.Id == id);

           }
       }

       public static List<DataCase> GetAllForDataset(int datasetId, bool testData)
       {
           if (datasetId == 0) return null;
           using (var context = new DomainDbContext())
           {
               var dataset = context.DataSets.FirstOrDefault(x => x.Id == datasetId);
               if (dataset == null) return null;
               var returnList = context.DataCases.Include(x=>x.Attributes).Include(x=>x.Attributes.Select(y=>y.AttributeType)).Include(x=>x.DataClass).Include(x=>x.DataSet).Where(x => x.DataSet.Id == datasetId).ToList();
               foreach(var c in returnList){
                   c.Attributes = c.Attributes.OrderBy(x => x.AttributeType.Position).ToList();
               }
               return returnList;
           }
       } 

       public static DataCase Add(int datasetId, int dataClassId)
       {
           using (var context = new DomainDbContext())
           {
               var dataset = context.DataSets.FirstOrDefault(x => x.Id == datasetId);
               var dataClass = context.DataClasses.FirstOrDefault(x => x.Id == dataClassId);
               if (dataset == null) return null;
               var addedObject = context.DataCases.Add(new DataCase()
               {
                   DataSet = dataset,
                   DataClass = dataClass
               });
               context.SaveChanges();
               return addedObject;
           }
       }
    }
}
