﻿using System;
using System.Collections.Generic;
using System.Linq;
using Domain.DTO;
using Domain.DTO.DialogueModels;
using Domain.Implementation.Master.Implementation;

namespace Domain.Implementation
{
    public class DialogueManager
    {
        static DialogueManager _manager;

        private static Stack<ICcbrAlgorithm> _goalStack;
        private static AttributeType _currentQuery;
        private static List<DataAttribute> _answers;
        private static DataSet _topLevelDialogue;

        public static DialogueManager GetInstance()
        {

            return _manager ?? (

                _manager = new DialogueManager()
                );
        }


        public DialogueStateModel StartDialogue()
        {
            _goalStack = new Stack<ICcbrAlgorithm>();
            _answers = new List<DataAttribute>();
            _topLevelDialogue = DatasetRepository.GetById(10);
            _goalStack.Push(new NaCoDAE(_topLevelDialogue, 4, 0, 2));
            var qa = _goalStack.Peek().InitializeDialog(_answers);
            _currentQuery = _topLevelDialogue.AttributeTypes.First();
            qa.Question = _topLevelDialogue.AttributeTypes.First();
            return qa;
        }

        public DialogueStateModel GetNextQuestion()
        {
            var nextQuestion = _goalStack.Peek().GetNextQuestion();
            var message = "";
            if (nextQuestion.Question == null)
            {
                if (_goalStack.Count == 1)
                {
                    _goalStack.Push(GetNewDialogueFromClass(nextQuestion.DataClass));
                    if(nextQuestion.DataClass.Name == "Fertility"){
                        message = "We can help you with a Fertility consultation, please answer the questions below";
                    }
                    else{
                        message = "We beleve your problems relate to: <strong>" + nextQuestion.DataClass.Name + "</strong>.<br/>Please answer the questions below";
                    }
                    InferAnswers();
                    nextQuestion = _goalStack.Peek().InitializeDialog(_answers);

                }
                else
                {
                    if (nextQuestion.DataClass.Id == 4)
                    {
                        _goalStack.Pop();
                        if (_goalStack.Count == 1)
                        {
                            message = "You do not have any unrinal infections. If you whant you can continue the dialogue by answering the new question below";
                            nextQuestion = _goalStack.Peek().GetNextQuestion();
                            var nextInLine = nextQuestion.ScoreObjects.OrderBy(x => x.Score).FirstOrDefault(x => x.Case.DataClass.Id != 4);
                                nextQuestion.DataClass = null;
                            if(nextInLine != null){
                                nextQuestion.DataClass = nextInLine.Case.DataClass;
                            }
                            if(nextQuestion.Question == null){
                                var answeredAttributes = _answers.Select(y => y.AttributeType);
                                nextQuestion.Question = _topLevelDialogue.AttributeTypes.FirstOrDefault(x => !answeredAttributes.Contains(x));
                            }
                        }
                        else
                        {
                            nextQuestion = _goalStack.Peek().GetNextQuestion();
                        }
                    }
                    else
                    {
                        message = "Your results are: " + nextQuestion.DataClass.Name;
                    }
                }
            }
            nextQuestion.Message = message;
            _currentQuery = nextQuestion.Question;

            return nextQuestion;
        }

        public void AddAnswerToQuestion(string value, int attributeTypeId)
        {
            var answer = new DataAttribute();
            if (_goalStack.Count > 0)
            {
                if (_currentQuery.Id == 171)
                {
                    if (value.Contains('-'))
                    {
                        var split = value.Split('-');
                        var age = split[0];
                        var sex = split[1];
                        answer = new DataAttribute
                        {
                            AttributeType = _currentQuery,
                            Value = age
                        };
                        _goalStack.Peek().SaveAnswer(answer);
                        _answers.Add(answer);
                        answer = new DataAttribute
                        {
                            AttributeType = _topLevelDialogue.AttributeTypes.FirstOrDefault(x => x.Id == 172),
                            Value = sex
                        };
                        _goalStack.Peek().SaveAnswer(answer);
                        _answers.Add(answer);
                        return;
                    }
                }

                answer = new DataAttribute
                {
                    AttributeType = _currentQuery,
                    Value = value
                };
                _goalStack.Peek().SaveAnswer(answer);
                _answers.Add(answer);
            }
        }

        public ICcbrAlgorithm GetNewDialogueFromClass(DataClass dc){
            
            var datset = DatasetRepository.GetAll().FirstOrDefault(x => x.Name == dc.Name);
            datset = DatasetRepository.GetById(datset.Id);
            if(datset != null){
                if(datset.AlgorithmType == 0){
                    return new NaCoDAE(datset, datset.RetrievelSet, datset.Alpa, 1, 1, 1);
                }
                else if(datset.AlgorithmType == 1){
                    return new INNk(datset, datset.RetrievelSet, (datset.Global ? "global" : "local"), 1, 1);
                }
            }

            return null;
        }

        public void InferAnswers()
        {

            var hurtQuestion = _answers.FirstOrDefault(x => x.AttributeType.Id == 174);

            if (hurtQuestion != null && hurtQuestion.Value == "Lumbar")
            {
                var lumbarAttr = AttributeTypeRepository.GetById(4);
                var c = new AttributeType { RelatedAttributes = new List<AttributeType>() };
                c.RelatedAttributes.Add(lumbarAttr);

                _answers.Add(new DataAttribute { Value = "true", AttributeType = c });
            }

        }

    }
}
