﻿using System;
using System.Web.Mvc;
using Domain.DTO.DialogueModels;
using Domain.Implementation;
using Domain.Implementation.Master.Implementation;
using Master.Models;

namespace Master.Controllers
{
    public class DatasetTestController : Controller
    {
        private static ICcbrAlgorithm _method;
        public ActionResult Index()
        {
            var viewModel = new DatasetViewModel();
            var datasets = DatasetRepository.GetAll();
            viewModel.DataSets = datasets;
            viewModel.Algorithms = (CcbrAlgorithms[])Enum.GetValues(typeof(CcbrAlgorithms));
            viewModel.SimilartyFunctions = (SimilartyFunctions[])Enum.GetValues(typeof(SimilartyFunctions));
            viewModel.AttributeSelectionFunctions = (AttributeSelectionFunctions[])Enum.GetValues(typeof(AttributeSelectionFunctions));
            viewModel.ThresholdFunctions = (ThresholdFunctions[])Enum.GetValues(typeof(ThresholdFunctions));
            viewModel.InferenceMethods = (InferenceMethods[])Enum.GetValues(typeof(InferenceMethods));

            return View(viewModel);
        }

        public ActionResult StartDataSet(int datasetId, int algorithm = 0, int thresholdFunc = 0, int simFunc = 0, int attrSelectFunc = 0, int k = 0, string localOrGlobal = "", int retrievalSetSize = 5, double thresholdValue = 0.4, int inferenceMethod = 0)
        {
            var dataSet = DatasetRepository.GetById(datasetId);

            switch (algorithm)
            {
                case 0:
                    _method = new NaCoDAE(dataSet, retrievalSetSize, thresholdValue, thresholdFunc, simFunc, attrSelectFunc, inferenceMethod);
                    break;
                case 1:
                    _method = new INNk(dataSet, k, localOrGlobal, simFunc, attrSelectFunc, inferenceMethod);
                    break;
                default:
                    _method = new NaCoDAE(dataSet);
                    break;
            }

            var result = _method.LeaveOneOutTest();

            return PartialView(result);
        }

        public ActionResult GetProgress()
        {
            return Json(_method != null ? _method.GetProgression() : 0);
        }
    }
}